package com.deviceatlas.examples;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;

import java.util.HashMap;
import java.util.Map;

/**
 * Supply Side Platform (SSP) CLI example of how HTTP headers can be parsed
 * to the OpenRTB SUA object according to the OpenRTB 2.6 specification SUA (Structured User Agent).
 *
 * @author DeviceAtlas Limited (c)
 */
public class SSPExample {

    private final Gson gson;

    public SSPExample() {
        gson = new GsonBuilder()
                .setPrettyPrinting()
                .disableHtmlEscaping()
                .create();
    }

    public static void main (String [] args) {
        SSPExample sspExample = new SSPExample();

        Map<String,String> headers = new HashMap<>();
        headers.put("Sec-Ch-Ua", "\"Google Chrome\";v=\"107\", \"Chromium\";v=\"107\", \"Not=\"A?Brand\";v=\"24\"");
        headers.put("Sec-Ch-Ua-Platform", "\"macOS\"");
        headers.put("Sec-Ch-Ua-Model", "");
        headers.put("Sec-Ch-Ua-Full-Version-List", "\"Google Chrome\";v=\"107.0.5304.87\", \"Chromium\";v=\"107.0.5304.87\", \"Not=\"A?Brand\";v=\"24.0.0.0\"");
        headers.put("Sec-Ch-Ua-Bitness", "\"64\"");
        headers.put("Sec-Ch-Ua-Mobile", "?1");
        headers.put("Sec-Ch-Ua-Arch", "\"x86\"");
        headers.put("Sec-Ch-Ua-Platform-Version", "\"13.0.0\"");

        String sua = sspExample.parseHeadersToSua(headers);
        System.out.println("SUA JSON Object:%n %s".formatted(sua));
    }

    /**
     *  Parse the headers to SUA object (OpenRTB-2-6 Documenation - 3.2.29 Object: UserAgent)
     *  which is part of the Device Object (OpenRTB-2-6 Documenation - 3.2.18 Object: Device)
     * @param headers
     * @return
     */
    private String parseHeadersToSua(Map<String, String> headers) {
        String headersAsJson = gson.toJson(headers);
        System.out.println("Headers:\n %s".formatted(headersAsJson));
        System.out.println("-------------------------------------------------------------");

        JsonObject suaObject  = new JsonObject();

        JsonArray browsers  = new JsonArray();
        String fullVersionList = headers.get("Sec-Ch-Ua-Full-Version-List");

        for (String fullVersion : fullVersionList.split(", ")){
            JsonObject browser  = new JsonObject();
            String [] brandAndVersion = fullVersion.split(";v=");
            browser.addProperty("brand", escapeBackslashAndDoubleQuote(removeBeginAndEndDoubleQuotes(brandAndVersion[0])));
            browser.add("version", parseToJsonVersionArray(removeBeginAndEndDoubleQuotes(brandAndVersion[1])));

            browsers.add(browser);
        }

        JsonObject platform  = new JsonObject();
        platform.addProperty("brand", escapeBackslashAndDoubleQuote(removeBeginAndEndDoubleQuotes(headers.get("Sec-Ch-Ua-Platform"))));
        platform.add("version", parseToJsonVersionArray(removeBeginAndEndDoubleQuotes(headers.get("Sec-Ch-Ua-Platform-Version"))));

        suaObject.add("browsers", browsers);
        suaObject.add("platform", platform);

        /**
         * The SUA mobile attribute expects the integer value 1 for true values
         * and 0 for false values as per documentation
         */
        if (headers.get("Sec-Ch-Ua-Mobile").equals("?1")){
            suaObject.addProperty("mobile", "1");
        }else {
            suaObject.addProperty("mobile", "0");
        }

        suaObject.addProperty("architecture", removeBeginAndEndDoubleQuotes(headers.get("Sec-Ch-Ua-Arch")));
        suaObject.addProperty("bitness", removeBeginAndEndDoubleQuotes(headers.get("Sec-Ch-Ua-Bitness")));
        suaObject.addProperty("model", removeBeginAndEndDoubleQuotes(headers.get("Sec-Ch-Ua-Model")));

        JsonObject root = new JsonObject();
        root.add("sua", suaObject);
        return gson.toJson(root);
    }

    /**
     * Remove the extra double quotes present initially in the header
     * @param value
     * @return
     */
    private String removeBeginAndEndDoubleQuotes(String value) {
        if (value.equals("")){
            return value;
        }

        return value.substring(1, value.length()-1);
    }

    /**
     * Parse the version to a version array E.g. from "102.0.5005.115" to ["102", "0", "5005", "115"]
     * @param versions
     * @return
     */
    private JsonArray parseToJsonVersionArray(String versions) {
        JsonArray versionsJsonArray  = new JsonArray();

        for (String version :  versions.split("\\.")){
            versionsJsonArray.add(version);
        }
        return versionsJsonArray;
    }

    /**
     * Some values need to be escaped as double-quotes and backslash might be present
     * @param value
     * @return
     */
    private String escapeBackslashAndDoubleQuote(String value) {
        return value.replace("/\\/", "\\\\").replace("\"","\\\"");
    }
}
